<?php

declare(strict_types = 1);

namespace Internetgalerie\IgFormlog\Controller\Backend;

/*
 * This file is part of the Internetgalerie TYPO3 IgFormlog project.
 */

use Internetgalerie\IgFormlog\Domain\FormLog\Filters;
use Internetgalerie\IgFormlog\Domain\Repository\FormLogEntryRepository;
use Internetgalerie\IgFormlog\Mvc\View\Export\CsvView;
use Internetgalerie\IgFormlog\Mvc\View\Export\XlsxView;
use Internetgalerie\IgFormlog\Mvc\View\FormatViewResolver;
use TYPO3\CMS\Backend\Routing\UriBuilder;
use TYPO3\CMS\Backend\View\BackendTemplateView;
use TYPO3\CMS\Core\Pagination\SimplePagination;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Pagination\QueryResultPaginator;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

use TYPO3\CMS\Form\Domain\Model\FormDefinition;
use TYPO3\CMS\Form\Domain\Model\FormElements\AbstractFormElement;
use TYPO3\CMS\Form\Domain\Factory\ArrayFormFactory;

/**
 * Controller for form log management
 */
class FormLogController extends ActionController
{
    /**
     * @var \Internetgalerie\IgFormlog\Mvc\View\ConfigurableViewInterface
     */
    protected $view = null;

    protected array $viewFormatToObjectNameMap = [
        'csv' => CsvView::class,
        'xlsx' => XlsxView::class,
    ];

    /**
     * @var string
     */
    protected $defaultViewObjectName = BackendTemplateView::class;

    protected FormLogEntryRepository $formLogEntryRepository;

    const EXCLUDE_FIELDS = ['Honeypot', 'StaticText', 'ContentElement', 'GridRow', 'SummaryPage', 'SrFreecap'];
 
    /**
     * @var \TYPO3\CMS\Form\Mvc\Persistence\FormPersistenceManagerInterface
     */
    protected $formPersistenceManager;

    
    public function injectFormLogEntryRepository(FormLogEntryRepository $formLogEntryRepository)
    {
        $this->formLogEntryRepository = $formLogEntryRepository;
    }

    /**
     * @param \TYPO3\CMS\Form\Mvc\Persistence\FormPersistenceManagerInterface $formPersistenceManager
     * @internal
     */
    public function injectFormPersistenceManager(\TYPO3\CMS\Form\Mvc\Persistence\FormPersistenceManagerInterface $formPersistenceManager)
    {
        $this->formPersistenceManager = $formPersistenceManager;
    }
    
    /**
     * Initialize all actions
     *
     * @return void
     */
    public function initializeAction()
    {
        if ($this->arguments->hasArgument('filters')) {
            $this->request->setArgument('filters', $this->request->hasArgument('filters') ? $this->request->getArgument('filters') : []);
            $filtersArgument = $this->arguments->getArgument('filters');
            $filtersArgument->getPropertyMappingConfiguration()
                ->allowAllProperties()
                ->forProperty('*')
                    ->allowAllProperties();
        }
    }

    /**
     * Main overview action
     *
     * @param Filters $filters
     * @param int $currentPageNumber
     * @return void
     */
    public function indexAction(Filters $filters, int $currentPageNumber = 1)
    {
        $entries = $this->formLogEntryRepository->findAllFiltered($filters);
        $paginator = new QueryResultPaginator($entries, $currentPageNumber);
        /** @var UriBuilder */
        $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);

        $this->view->assignMultiple([
            'entries' => $paginator->getPaginatedItems(),
            'entriesCount' => count($entries),
            'filters' => $filters,
            'pagination' => new SimplePagination($paginator),
            'currentPageNumber' => $currentPageNumber,
            'dateFormat' => $this->settings['dateTimeFormat'] ?: \DateTime::W3C,
            'isoDateFormat' => \DateTime::W3C,
            'daterangepickerTranslations' => $this->prepareDaterangepickerTranslations(),
            'inlineSettings' => [
                'ig_formlog' => [
                    'suggestUri' => (string)$uriBuilder->buildUriFromRoute('ajax_ig_formlog_suggest'),
                    'language' => $GLOBALS['BE_USER']->uc['lang'],
                    'timeZone' => date_default_timezone_get(),
                ],
            ],
        ]);
    }

    public function initializeExportAction(): void
    {
        $viewResolver = GeneralUtility::makeInstance(FormatViewResolver::class);

        foreach ($this->viewFormatToObjectNameMap as $format => $viewClassName) {
            $viewResolver->map($format, $viewClassName);
        }

        $this->injectViewResolver($viewResolver);
    }

    /**
     * Export CSV of form log entries
     *
     * @param Filters $filters
     * @return void
     */
    public function exportAction(Filters $filters)
    {
        $now = new \DateTime();
        $fileBasename = sprintf('ig_formlog-%s', $now->format('Y-m-d-H-i-s'));
	/* IG ab hier */
        $columns = $this->settings['export']['columns'];
 
        if($filters->getIdentifier()) {
            $formIdentifier = $filters->getIdentifier()->getValue();
	    // formIdentifier enthaelt bei TYPO3 9.5 die tt_content UID
            if(\TYPO3\CMS\Core\Utility\VersionNumberUtility::convertVersionNumberToInteger(\TYPO3\CMS\Core\Utility\VersionNumberUtility::getNumericTypo3Version()) < 10000000) {
                $formIdentifier = substr($formIdentifier,0,strrpos($formIdentifier,'-'));
            }
            if($formIdentifier) {
                $formPersistenceIdentifier ='';
                foreach ($this->formPersistenceManager->listForms() as $formDefinition) {
                    if($formDefinition['identifier']==$formIdentifier) {
                       $formPersistenceIdentifier = $formDefinition['persistenceIdentifier'];
                    }
                }
                if($formPersistenceIdentifier) {
                //$formPersistenceIdentifier = 'Materialbestellung';
                    $formDefinition = $this->getFormDefinitionObject($formPersistenceIdentifier, true);
                    $columns = $this->getColumns($formDefinition);
                    //var_dump($columns);exit(0);
                }
            }
        }
 
        $this->view->setConfiguration([
            'columns' => $columns, /*IG BIS HIER */
            'dateTimeFormat' => $this->settings['dateTimeFormat'],
            'fileBasename' => $fileBasename,
        ]);
        $this->view->assign('items', $this->formLogEntryRepository->findAllFiltered($filters));
    }

    /**
     * Prepare localized daterangepicker labels
     *
     * @return array
     */
    protected function prepareDaterangepickerTranslations(): array
    {
        $translationIdentifiers = [
            'labels' => [
                'applyButtonTitle',
                'cancelButtonTitle',
                'startLabel',
                'endLabel',
            ],
            'ranges' => [
                'last30days',
                'lastYear',
                'other',
            ],
            'periods' => [
                'day',
                'week',
                'month',
                'quarter',
                'year',
            ],
        ];
        $translations = [];

        foreach ($translationIdentifiers as $section => $identifiers) {
            foreach ($identifiers as $identifier) {
                $translations[$section][$identifier] = LocalizationUtility::translate('ig_formlog.daterangepicker.' . $section . '.' . $identifier, 'IgFormlog');
            }
        }

        return $translations;
    }
    /**
     * Gets a form definition by a persistence form identifier
     *
     * @param string $formPersistenceIdentifier
     * @param bool $useFieldStateDataAsRenderables
     * @return FormDefinition
     * @throws RenderingException
     */
    protected function getFormDefinitionObject(
        string $formPersistenceIdentifier,
        $useFieldStateDataAsRenderables = false
    ): FormDefinition {
        $configuration = $this->formPersistenceManager->load($formPersistenceIdentifier);
        //$configuration = $this->getFormDefinition($formPersistenceIdentifier, $useFieldStateDataAsRenderables);
        /** @var ArrayFormFactory $arrayFormFactory */
        $arrayFormFactory = GeneralUtility::makeInstance(ArrayFormFactory::class);
        return $arrayFormFactory->build($configuration);
    }
 
 
    /**
     * Gets an array of all form renderables (recursive) by a form definition
     *
     * @param FormDefinition $formDefinition
     * @return array
     */
    protected function getColumns(FormDefinition $formDefinition): array
    {
 
        $columns = [];
        /** @var AbstractFormElement $renderable */
        foreach ($formDefinition->getRenderablesRecursively() as $renderable) {
            //echo($renderable->getType() .'<br />');
            if ($renderable instanceof AbstractFormElement && in_array($renderable->getType(), self::EXCLUDE_FIELDS, true) === false) {
                $columns[] = [
                    'property' => 'data.' . $renderable->getIdentifier(),
                    'label' => $renderable->getLabel(),
                ];
            }
        }
        return $columns;
    }
}
